<?php
/*
   Plugin Name: API de paiement en ligne DIGIPAY
   Description: DIGIPAY vous offre un API de paiement en ligne hautement securisé facile d'utilisation.
   Version: 1.1.1
   Author: DIGIPAY
   Author URI: https://bgdigit-all.com
 */
if (!defined('ABSPATH')) {
  exit;
}

if (!in_array('woocommerce/woocommerce.php', apply_filters('active_plugins', get_option('active_plugins')))) {
  exit;
}




add_action('plugins_loaded', 'woocommerce_digipay_init', 0);

function woocommerce_digipay_init() {
  if (!class_exists('WC_Payment_Gateway'))
    return;

  class WC_Digipay extends WC_Payment_Gateway {

    public function __construct() {
      $this->digipay_errors = new WP_Error();
      /**
       * IPN
       */

      //id de la passerelle
      $this->id = 'digipay';
      $this->medthod_title = 'DigiPay';
      $this->icon = apply_filters('woocommerce_digipay_icon', plugins_url('assets/images/digipay.svg', __FILE__));
      $this->has_fields = false;
      //charger les champs pour paramètres de la passerelle.
      $this->init_form_fields();
      $this->init_settings();
      $this->title = $this->settings['title'];
      $this->description = $this->settings['description'];
      //Mes parametres
      $this->app_key =@$this->settings['app_key'];
      $this->secrete_key =@$this->settings['secrete_key'];
      $this->public_key =@$this->settings['public_key'];
      $this->env  = @$this->settings['env'];
      $this->status_after_payment  = @$this->settings['status_after_payment'];

      $this->digipay_host= 'https://proxy.bgpay.digital/';
      $this->posturl = $this->digipay_host.'api/payment/request';
      $this->submit_id = 0;
      //Transaction annulée
      $annulation =$success= $complet =-1;
      if(isset($_GET['cancel']) && $_GET['cancel']==1 )
        $annulation=1;
      if(isset($_GET['success']) && $_GET['success']==1 )
        $success = 1;


      if($annulation===1)
      {
        $message_type='error';
        $message='La transaction a été annulée';
        $wc_order_id = WC()->session->get('digipay_wc_oder_id');
        $order = new WC_Order($wc_order_id);
        $order->add_order_note($message);
        $rdi_url =$order->get_cancel_order_url();
      }
      if($success==1){
        $message="Paiement effectué avec succès.Merci d'avoir choisi Digipay";
        $message_type='success';
        $wc_order_id = WC()->session->get('digipay_wc_oder_id');
        $order = new WC_Order($wc_order_id);
        $order->add_order_note($message);
        $order->payment_complete();
        $rdi_url=$this->get_return_url($order);
      }

      if($success == 1|| $annulation == 1){
        $notification_message = array(
          'message' => $message,
          'message_type' => $message_type
        );
        if (version_compare(WOOCOMMERCE_VERSION, "2.2") >= 0) {
          $hash = WC()->session->get('digipay_wc_hash_key');
          add_post_meta($wc_order_id, '_digipay_hash', $hash, true);
        }
        update_post_meta($wc_order_id, '_digipay_wc_message', $notification_message);

        WC()->session->__unset('digipay_wc_hash_key');
        WC()->session->__unset('digipay_wc_order_id');

        wp_redirect($rdi_url);
        exit;
      }


      if (version_compare(WOOCOMMERCE_VERSION, '2.0.0', '>=')) {
        add_action('woocommerce_update_options_payment_gateways_' . $this->id, array(&$this, 'process_admin_options'));
      } else {
        add_action('woocommerce_update_options_payment_gateways', array(&$this, 'process_admin_options'));
      }
    }





    function init_form_fields() {
      $this->form_fields = array(
        'enabled' => array(
          'title' => __('Activer/Désactiver', 'digipay'),
          'type' => 'checkbox',
          'label' => __('Activation du module de paiement DigiPay.', 'digipay'),
          'default' => 'no'),
        'title' => array(
          'title' => __('Titre:', 'digipay'),
          'type' => 'text',
          'description' => __('Texte que verra le client lors du paiement de sa commande.', 'digipay'),
          'default' => __('Paiement avec DigiPay', 'digipay')),
        'description' => array(
          'title' => __('Description:', 'digipay'),
          'type' => 'textarea',
          'description' => __('Description que verra le client lors du paiement de sa commande.', 'digipay'),
          'default' => __('Aggregateur de paiement developpé par BG Digital. <br /><img alt="Paiement avec  Orange Money | Free Money | E Money | Wizall | Yup | MasterCard | Visa | Airtel | MTN Momo | Paypal | Digicash" src="wp-content/plugins/digipay-wc/assets/images/Logo-API-.svg" width="100%">', 'digipay')),

        'app_key' => array(
          'title' => __("Clé de l'application", 'digipay'),
          'type' => 'text',
          'description' => __("Identifiant de l'application lié á la boutique en ligne")),
        'secret_key_prod' => array(
          'title' => __("Clé privée de Production", 'digipay'),
          'type' => 'text',
          'description' => __('Clé privée de production de sécurité á voir dans l\'espace client partie application')),

        'public_key_prod' => array(
          'title' => __("Clé public  de Production", 'digipay'),
          'type' => 'text',
          'description' => __('Clé  public de Production  á voir dans l\'espace client partie application')),

        'secret_key_test' => array(
          'title' => __("Clé privée de Test", 'digipay'),
          'type' => 'text',
          'description' => __('Clé privée de Test  á voir dans l\'espace client partie application')),

        'public_key_test' => array(
          'title' => __("Clé public  de Test", 'digipay'),
          'type' => 'text',
          'description' => __('Clé  public de test á voir dans l\'espace client partie application')),
        'env' => array(
          'title' => __("Environnement", 'digipay'),
          'description' => __('L\'environement de l\'application. mode teste ou production'),
          'css'=>'padding:0%;',
          'type' => 'select',
          'options'=>array('test'=>'Test','prod' => 'Production'),
        ),
        'devise' => array(
          'title' => __("Dévise", 'digipay'),
          'description' => __('Choisir une dévise.'),
          'css'=>'padding:0%;',
          'type' => 'select',
          'options'=>$this->getcurren(),
        ),

        'status_after_payment' => array(
          'title' => __("Status apres paiement", 'digipay'),
          'description' => __('Choisir une action.'),
          'css'=>'padding:0%;',
          'type' => 'select',
          'options'=>['completed'=>'Termine','processing'=>'En attende'],
        ),



      );
    }
    public function getcurren(){

      $rest =  json_decode("[{\"XOF\":\"XOF\"},{\"EUR\":\"EUR\"},{\"USD\":\"USD\"},{\"XDR\":\"XDR\"},{\"AED\":\"AED\"},{\"AFN\":\"AFN\"},{\"ALL\":\"ALL\"},{\"AMD\":\"AMD\"},{\"ANG\":\"ANG\"},{\"AOA\":\"AOA\"},{\"ARS\":\"ARS\"},{\"AUD\":\"AUD\"},{\"AWG\":\"AWG\"},{\"AZN\":\"AZN\"},{\"BAM\":\"BAM\"},{\"BBD\":\"BBD\"},{\"BDT\":\"BDT\"},{\"BGN\":\"BGN\"},{\"BHD\":\"BHD\"},{\"BIF\":\"BIF\"},{\"BMD\":\"BMD\"},{\"BND\":\"BND\"},{\"BOB\":\"BOB\"},{\"BRL\":\"BRL\"},{\"BSD\":\"BSD\"},{\"BTC\":\"BTC\"},{\"BTN\":\"BTN\"},{\"BWP\":\"BWP\"},{\"BYN\":\"BYN\"},{\"BZD\":\"BZD\"},{\"CAD\":\"CAD\"},{\"CDF\":\"CDF\"},{\"CHF\":\"CHF\"},{\"CLF\":\"CLF\"},{\"CLP\":\"CLP\"},{\"CNH\":\"CNH\"},{\"CNY\":\"CNY\"},{\"COP\":\"COP\"},{\"CRC\":\"CRC\"},{\"CUC\":\"CUC\"},{\"CUP\":\"CUP\"},{\"CVE\":\"CVE\"},{\"CZK\":\"CZK\"},{\"DJF\":\"DJF\"},{\"DKK\":\"DKK\"},{\"DOP\":\"DOP\"},{\"DZD\":\"DZD\"},{\"EGP\":\"EGP\"},{\"ERN\":\"ERN\"},{\"ETB\":\"ETB\"},{\"FJD\":\"FJD\"},{\"FKP\":\"FKP\"},{\"GBP\":\"GBP\"},{\"GEL\":\"GEL\"},{\"GGP\":\"GGP\"},{\"GHS\":\"GHS\"},{\"GIP\":\"GIP\"},{\"GMD\":\"GMD\"},{\"GNF\":\"GNF\"},{\"GTQ\":\"GTQ\"},{\"GYD\":\"GYD\"},{\"HKD\":\"HKD\"},{\"HNL\":\"HNL\"},{\"HRK\":\"HRK\"},{\"HTG\":\"HTG\"},{\"HUF\":\"HUF\"},{\"IDR\":\"IDR\"},{\"ILS\":\"ILS\"},{\"IMP\":\"IMP\"},{\"INR\":\"INR\"},{\"IQD\":\"IQD\"},{\"IRR\":\"IRR\"},{\"ISK\":\"ISK\"},{\"JEP\":\"JEP\"},{\"JMD\":\"JMD\"},{\"JOD\":\"JOD\"},{\"JPY\":\"JPY\"},{\"KES\":\"KES\"},{\"KGS\":\"KGS\"},{\"KHR\":\"KHR\"},{\"KMF\":\"KMF\"},{\"KPW\":\"KPW\"},{\"KRW\":\"KRW\"},{\"KWD\":\"KWD\"},{\"KYD\":\"KYD\"},{\"KZT\":\"KZT\"},{\"LAK\":\"LAK\"},{\"LBP\":\"LBP\"},{\"LKR\":\"LKR\"},{\"LRD\":\"LRD\"},{\"LSL\":\"LSL\"},{\"LYD\":\"LYD\"},{\"MAD\":\"MAD\"},{\"MDL\":\"MDL\"},{\"MGA\":\"MGA\"},{\"MKD\":\"MKD\"},{\"MMK\":\"MMK\"},{\"MNT\":\"MNT\"},{\"MOP\":\"MOP\"},{\"MRO\":\"MRO\"},{\"MRU\":\"MRU\"},{\"MUR\":\"MUR\"},{\"MVR\":\"MVR\"},{\"MWK\":\"MWK\"},{\"MXN\":\"MXN\"},{\"MYR\":\"MYR\"},{\"MZN\":\"MZN\"},{\"NAD\":\"NAD\"},{\"NGN\":\"NGN\"},{\"NIO\":\"NIO\"},{\"NOK\":\"NOK\"},{\"NPR\":\"NPR\"},{\"NZD\":\"NZD\"},{\"OMR\":\"OMR\"},{\"PAB\":\"PAB\"},{\"PEN\":\"PEN\"},{\"PGK\":\"PGK\"},{\"PHP\":\"PHP\"},{\"PKR\":\"PKR\"},{\"PLN\":\"PLN\"},{\"PYG\":\"PYG\"},{\"QAR\":\"QAR\"},{\"RON\":\"RON\"},{\"RSD\":\"RSD\"},{\"RUB\":\"RUB\"},{\"RWF\":\"RWF\"},{\"SAR\":\"SAR\"},{\"SBD\":\"SBD\"},{\"SCR\":\"SCR\"},{\"SDG\":\"SDG\"},{\"SEK\":\"SEK\"},{\"SGD\":\"SGD\"},{\"SHP\":\"SHP\"},{\"SLL\":\"SLL\"},{\"SOS\":\"SOS\"},{\"SRD\":\"SRD\"},{\"SSP\":\"SSP\"},{\"STD\":\"STD\"},{\"STN\":\"STN\"},{\"SVC\":\"SVC\"},{\"SYP\":\"SYP\"},{\"SZL\":\"SZL\"},{\"THB\":\"THB\"},{\"TJS\":\"TJS\"},{\"TMT\":\"TMT\"},{\"TND\":\"TND\"},{\"TOP\":\"TOP\"},{\"TRY\":\"TRY\"},{\"TTD\":\"TTD\"},{\"TWD\":\"TWD\"},{\"TZS\":\"TZS\"},{\"UAH\":\"UAH\"},{\"UGX\":\"UGX\"},{\"UYU\":\"UYU\"},{\"UZS\":\"UZS\"},{\"VEF\":\"VEF\"},{\"VES\":\"VES\"},{\"VND\":\"VND\"},{\"VUV\":\"VUV\"},{\"WST\":\"WST\"},{\"XAF\":\"XAF\"},{\"XAG\":\"XAG\"},{\"XAU\":\"XAU\"},{\"XCD\":\"XCD\"},{\"XPD\":\"XPD\"},{\"XPF\":\"XPF\"},{\"XPT\":\"XPT\"},{\"YER\":\"YER\"},{\"ZAR\":\"ZAR\"},{\"ZMW\":\"ZMW\"},{\"ZWL\":\"ZWL\"}]",true);
      $return=[];
      foreach ($rest as $one){

        foreach($one as $key){
          $return[$key] =   $key;break;
        }
      }
      return $return ;
    }

    public function admin_options() {
      echo '<h3>' . __('Passerelle de paiement DigiPay', 'digipay') . '</h3>';
      echo '<p>' . __('DigiPay est la meilleure plateforme de paiement en ligne.') . '</p>';
      echo '<table class="form-table">';

      $this->generate_settings_html();
      echo '</table>';
    }

    function payment_fields() {
      if ($this->description)
        echo wpautop(wptexturize($this->description));
    }

    protected function get_digipay_args($order, $order_id) {

      global $woocommerce;

      $txnid = $order->id . '_' . date("ymds");

      $redirect_url = $woocommerce->cart->get_checkout_url();

      $productinfo = "Commande: " . $order->id;

      $str = "$this->merchant_id|$txnid|$order->order_total|$productinfo|$order->billing_first_name|$order->billing_email|||||||||||$this->salt";
      $hash = hash('sha512', $str);

      WC()->session->set('digipay_wc_hash_key', $hash);

      $items = $woocommerce->cart->get_cart();
      $produit="";
      foreach ($items as $item) {
        $produit=$produit.$item["data"]->post->post_title." ";

      }

      $itemsId = [];
      foreach ($order->get_items() as $item_id => $item) {
        array_push($itemsId, $item_id);
      }

     // $opt = get_settings('woocommerce_digipay_settings', array() );
      $opt = get_option('woocommerce_digipay_settings', array() );

      $cancel_url=  strpos($redirect_url,"?") === false ? $redirect_url.'?cancel=1' : $redirect_url.'&cancel=1' ;
      $success_url=  strpos($redirect_url,"?") === false ? $redirect_url.'?success=1' : $redirect_url.'&success=1' ;
      $postfields = array(
        "amount"   => $order->order_total,
        "currency"       =>   $opt['devise'],  //"xof",
        "app_key"       => $opt['app_key'],  //"xof",
        "secrete_key"       => $opt['env'] ==='prod' ? $opt['secret_key_prod'] : $opt['secret_key_test'],  //"xof",
        "public_key"       => $opt['env'] ==='prod' ? $opt['public_key_prod'] : $opt['public_key_test'],  //"xof",
        //  "no_calculate_fee" => $opt['fee'],
        "ref_commande"  =>$order->id.'_'.time(),
        "commande_name" =>"Achat " . $order->order_total . " ".$opt['devise']." sur " . get_bloginfo("name"),
        "mode"          => $opt['env'],
        "success_url" => $success_url,
        "ipn_url"=>  get_site_url(null,'','https')  ."/?wp_digipay_digital_ipn=1",
        "cancel_url"   => $cancel_url,
        "failed_url"   =>$cancel_url,
        "data_transactions"=> json_encode([
          'order' => $order_id,
          'order_id' => $order->get_id(),
          'order_number' => $order->get_order_number(),
            'order_key_value' => version_compare(WC_VERSION, '3.0', '<') ? $order->order_key : $order->get_order_key()
        ]));
      //fin dame arguments

      apply_filters('woocommerce_digipay_args', $postfields, $order);

      return $postfields;
    }


    function post($url, $data, $order_id,$header = [])
    {

      $strPostField = json_encode($data);
      $ch = curl_init($url);
      curl_setopt($ch, CURLOPT_CUSTOMREQUEST, "POST");
      curl_setopt($ch, CURLOPT_POSTFIELDS, $strPostField);
      curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
      curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
      curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
      curl_setopt($ch, CURLOPT_HTTPHEADER, array_merge($header, [
        'Content-Type: application/json'
      ]));

      $response = curl_exec($ch);


      $jsonResponse = json_decode($response, true);
      WC()->session->set('digipay_wc_oder_id', $order_id);
      if(array_key_exists('error', $jsonResponse) && $jsonResponse['error']===false)
      {

        $url = $jsonResponse['data']['url_payment'] ;
        return $url ;
      }
      else {
        //var_dump($jsonResponse);die;
        wc_add_notice($jsonResponse['msg'], "error");
        foreach ($jsonResponse['data'] as $message){
          wc_add_notice($message, "error");
        }
        return '';
      }
    }

    function process_payment($order_id) {
      $order = new WC_Order($order_id);
      return array(
        'result' => 'success',
        'redirect' => $this->post($this->posturl, $this->get_digipay_args($order, $order_id), $order_id,[

        ])
      );
    }

    function get_pages($title = false, $indent = true) {
      $wp_pages = get_pages('sort_column=menu_order');
      $page_list = array();
      if ($title)
        $page_list[] = $title;
      foreach ($wp_pages as $page) {
        $prefix = '';
        if ($indent) {
          $has_parent = $page->post_parent;
          while ($has_parent) {
            $prefix .= ' - ';
           // $next_page = get_page($has_parent);
            $next_page = get_post($has_parent);
            $has_parent = $next_page->post_parent;
          }
        }
        $page_list[$page->ID] = $prefix . $page->post_title;
      }
      return $page_list;
    }


    static function add_digipay_fcfa_currency($currencies) {
      $currencies['FCFA'] = __('BCEAO XOF', 'woocommerce');
      return $currencies;
    }

    static function add_digipay_fcfa_currency_symbol($currency_symbol, $currency) {
      switch (
      $currency) {
        case 'FCFA': $currency_symbol = 'FCFA';
          break;
      }
      return $currency_symbol;
    }

    static function woocommerce_add_digipay_gateway($methods) {
      $methods[] = 'WC_Digipay';
      return $methods;
    }

    // Add settings link on plugin page
    static function woocommerce_add_digipay_settings_link($links) {
      $settings_link = '<a href="admin.php?page=wc-settings&tab=checkout&section=wc_digipay">Paramètres</a>';
      array_unshift($links, $settings_link);
      return $links;
    }


  }



  $plugin = plugin_basename(__FILE__);

  add_filter('woocommerce_currencies', array('WC_Digipay', 'add_digipay_fcfa_currency'));
  add_filter('woocommerce_currency_symbol', array('WC_Digipay', 'add_digipay_fcfa_currency_symbol'), 10, 2);

  add_filter("plugin_action_links_$plugin", array('WC_Digipay', 'woocommerce_add_digipay_settings_link'));
  add_filter('woocommerce_payment_gateways', array('WC_Digipay', 'woocommerce_add_digipay_gateway'));
  function digipay_ipn(){
  global $woocommerce;
    if (!empty($_GET['wp_digipay_digital_ipn']) && $_GET['wp_digipay_digital_ipn'] == '1'){
	    try {
        $options = get_option('woocommerce_digipay_settings', array());//DATA SENDED
        $mode = @$_POST['mode'];
        $hash_app_key = @$_POST['hash_app_key'];
        $hash_secret_key = @$_POST['hash_secret_key'];
        $hash_public_key = @$_POST['hash_public_key'];//OWNER DATA

        $my_hash_app_key = hash('sha512', @$options['app_key']);
        $my_hash_secret_key = '';
        $my_hash_public_key = '';
        if ($mode === 'prod') {
          $my_hash_secret_key = hash('sha512', $options['secret_key_prod']);
          $my_hash_public_key = hash('sha512', $options['public_key_prod']);
        } else {
          $my_hash_secret_key = hash('sha512', $options['secret_key_test']);
          $my_hash_public_key = hash('sha512', $options['public_key_test']);
        }
        if ($my_hash_public_key === $hash_public_key && $my_hash_secret_key === $hash_secret_key && $my_hash_app_key === $hash_app_key) {
          global $woocommerce;
          ini_set('display_errors', 1);
          error_reporting(E_ALL);
         $custom = json_decode(stripslashes($_POST['data_transactions']),true);
          $order_id = $custom['order'];
          global $wpdb;
//           $prefix = $wpdb->base_prefix;
//           $query = "UPDATE " . $prefix . "posts SET `post_status` = 'wc-completed' WHERE ID =". $order_id;
//           $wpdb->query($query);
         $order = new WC_Order($order_id);
            if(in_array($order->get_status(), ['completed'])){
             die('deja valider: ' . $order->get_status());
           }
          $msg = 'Paiement valider via API Digipay. Mode de paiement par'. @$_POST['payment_method'];
          $order->payment_complete();
          $order->update_status(@$options['status_after_payment'] ?: 'wc-completed');
          $order->add_order_note($msg);
          $notification_message = array(
                                  'message' => $msg,
                                  'message_type' => 'success'
                              );

                              if (version_compare(WOOCOMMERCE_VERSION, "2.2") >= 0) {
                                  add_post_meta($order_id, 'ref_bg_pay', @$_POST['ref_internal_digi_pa'], true);
                              }
         update_post_meta($order_id, 'bg_pay_notification_message', $notification_message);
          try {
                                  if (
                                      !@empty(@WC()->mailer()) &&
                                      !@(@WC()->mailer()->emails) &&
                                      !@empty(@WC()->mailer()->emails['WC_Email_New_Order']) &&
                                      @is_callable(@WC()->mailer()->emails['WC_Email_New_Order']->trigger)
                                  ) {
                                      @WC()->mailer()->emails['WC_Email_New_Order']->trigger($order_id, $order);
                                  }

                                  if (
                                      !@empty(@WC()->mailer()) &&
                                      !@(@WC()->mailer()->emails) &&
                                      !@empty(@WC()->mailer()->emails['WC_Email_Customer_Processing_Order']) &&
                                      @is_callable(@WC()->mailer()->emails['WC_Email_Customer_Processing_Order']->trigger)
                                  ) {
                                      @WC()->mailer()->emails['WC_Email_Customer_Processing_Order']->trigger($order_id, $order);
                                  }

                              } catch (Exception $e) {
                                  var_dump($e->getMessage());
                              }
        var_dump($_POST,$order);die;
        } else {
          die("NO_SUCCES");
        }
      } catch (Exception $e) {
        echo $e->getMessage();
        die('FAILED_ERROR'); 
      }

    }

  };

   add_action('init', 'digipay_ipn', 10, 0);

}


